###############################################################################
#                               GPRename
###############################################################################
# copyright (C) 2001-2004 Tristesse <tristesse at users.sourceforge.net>
#						  Zurd <zurd at users.sourceforge.net>
# Distributed under the terms of the GPL v2
###############################################################################
# Version 1.24
###############################################################################
# A GUI batch file renamer based on Gtk-Perl.
###############################################################################

$install_dir = $ARGV[0] . "/";

require $install_dir . "langs.pl";

######################################
# initial
######################################

use Gtk;
use Cwd;    #to get the current directory
use Fcntl ':flock';
use strict;
no strict "vars";

set_locale Gtk;
init Gtk;

my $false = 0;
my $true  = 1;

my $home_dir = "$ENV{'HOME'}" . "/";
my %settings;
my $setting_file = $home_dir . ".gprename";

&read_settings;
if ( $settings{'lang'} eq undef || $langs{$settings{'lang'}} eq undef) {
	$settings{'lang'} = 'English';
}
require $install_dir . 'languages/' . $langs{$settings{'lang'}};

my $root_dir  = "/";
my $start_dir = $ARGV[1];
$start_dir =~ s/\A\///;
$start_dir =~ s/\/\Z//;
my @start_path = reverse split( /\//, $start_dir );
my $initial_expand = @start_path;
$start_dir = '/' . $start_dir;
my @titles;    #for files
my @titles;    #for directory
my $path;
my %files_to_rename;
my %new_names;
my $preview_status = $false;
my %undo;

my $autoselected_item;
my $about_dialog;
my $prompt_restart_dialog;

# Create a window
my $window = new Gtk::Window('toplevel');
$window->set_default_size( 640, 480 );
$window->set_position('center');
$window->set_title("GPRename");
$window->border_width(5);
$window->signal_connect( "delete_event", \&quit );
$window->show();

# Set window icon
my $style;
my $pixmap;
my $mask;
my $icon = $install_dir . "pixmaps/gprename_icon64.xpm";
$style = $window->get_style()->bg('normal');
( $pixmap, $mask ) =
  Gtk::Gdk::Pixmap->create_from_xpm( $window->window, $style, $icon );
$window->window->set_icon( $window->window, $pixmap, $mask );

# Create a vertical main pane
my $main_paned = new Gtk::VPaned();
$window->add($main_paned);
$main_paned->set_handle_size(10);
$main_paned->set_gutter_size(20);
$main_paned->show();

# Create a vbox for menubar and pane
my $top_vbox = new Gtk::VBox( $false, 0 );
$main_paned->pack1( $top_vbox, $true, $true );
$top_vbox->show();

######################################
# MenuBar
######################################

# Create menubar
my $menubar = new Gtk::MenuBar();
$top_vbox->pack_start( $menubar, $false, $false, 0 );
$menubar->show();

# File
my $file_mm = new Gtk::MenuItem($str_File);
$menubar->append($file_mm);
$file_mm->show();

my $file_menu = new Gtk::Menu();
$file_mm->set_submenu($file_menu);

# File - Quit
my $quit_mi = new Gtk::MenuItem($str_Quit);
$file_menu->append($quit_mi);
$quit_mi->signal_connect( "activate", \&quit );
$quit_mi->show();

# Language
my $lang_mm = new Gtk::MenuItem($str_Lang);
$menubar->append($lang_mm);
$lang_mm->show();

my $lang_menu = new Gtk::Menu();
$lang_mm->set_submenu($lang_menu);

my $lang_rmi;
foreach $lang ( sort keys %langs ) {
	$lang_rmi = new Gtk::RadioMenuItem( $lang, $lang_rmi );
	$lang_menu->append($lang_rmi);
	if ( $lang eq $settings{'lang'} ) {
		$lang_rmi->set_active($true);
	}
	$lang_rmi->signal_connect( 'activate', \&language_selected, $lang );
	$lang_rmi->show();
}

# Help
my $help_mm = new Gtk::MenuItem($str_Help);
$menubar->append($help_mm);
$help_mm->show();

my $help_menu = new Gtk::Menu();
$help_mm->set_submenu($help_menu);

# Help - About
my $about_mi = new Gtk::MenuItem($str_About);
$help_menu->append($about_mi);
$about_mi->signal_connect( 'activate', \&help_about );
$about_mi->show();

######################################
# Monitor
######################################

# Create a horizontal pane
my $pane = new Gtk::HPaned();
$top_vbox->pack_end( $pane, $true, $true, 2 );
$pane->set_handle_size(10);
$pane->set_gutter_size(20);
$pane->show();

## Create the Tree Scrolled Window
my $vbox = new Gtk::VBox( $false, 0 );
$pane->add1($vbox);
$vbox->show();

# Create a ScrolledWindow for the tree
my $tree_scrolled_win = new Gtk::ScrolledWindow( undef, undef );
$tree_scrolled_win->set_usize( 200, 100 );
$vbox->pack_start( $tree_scrolled_win, $true, $true, 0 );
$tree_scrolled_win->set_policy( 'automatic', 'automatic' );
$tree_scrolled_win->show();

# Create a ScrolledWindow for the list
my $list_scrolled_win = new Gtk::ScrolledWindow( undef, undef );
$list_scrolled_win->set_policy( 'automatic', 'automatic' );
$list_scrolled_win->show();

# Create list box for files
@titles = ( $str_Filename, $str_Newname );
my $list = new_with_titles Gtk::CList(@titles);
$list_scrolled_win->add($list);
$list->set_column_width( 0, 240 );
$list->set_column_width( 1, 240 );
$list->set_selection_mode('extended');
$list->set_shadow_type('none');
$list->show();

# Create the File tab
my $notebook2 = new Gtk::Notebook();
$pane->add2($notebook2);
$file_label = new Gtk::Label($str_File);
$notebook2->append_page( $list_scrolled_win, $file_label );

# Create a ScrolledWindow for the Directory list
my $list_scrolled_win2 = new Gtk::ScrolledWindow( undef, undef );
$list_scrolled_win2->set_policy( 'automatic', 'automatic' );
$list_scrolled_win2->show();

# Create list box for the Directory list
@titles2 = ( $str_Filename, $str_Newname );
my $list2 = new_with_titles Gtk::CList(@titles2);
$list_scrolled_win2->add($list2);
$list2->set_column_width( 0, 250 );
$list2->set_column_width( 1, 250 );
$list2->set_selection_mode('extended');
$list2->set_shadow_type('none');
$list2->show();

# Create the Directory tab
$dir_label = new Gtk::Label($str_Directory);
$notebook2->append_page( $list_scrolled_win2, $dir_label );
$notebook2->show();

# Create root tree
my $tree = new Gtk::Tree();
$tree_scrolled_win->add_with_viewport($tree);
$tree->set_selection_mode('single');
$tree->set_view_mode('item');
$tree->show();

# Create root tree item widget
my $leaf = new_with_label Gtk::TreeItem($root_dir);
$tree->append($leaf);
$leaf->signal_connect( 'select', \&select_item, $root_dir );
$leaf->set_user_data($root_dir);
$leaf->show();

# Create the subtree
if ( has_sub_trees($root_dir) ) {
	my $subtree = new Gtk::Tree();
	$leaf->set_subtree($subtree);
	$leaf->signal_connect( 'expand', \&expand_tree, $subtree );
	$leaf->signal_connect( 'collapse', \&collapse_tree );
	$leaf->expand();
}

######################################
# Create manipulation items
######################################

my $table = new Gtk::Table( 1, 5, $true );
$main_paned->pack2( $table, $false, $true );
$table->show();

my $notebook = new Gtk::Notebook();
$table->attach_defaults( $notebook, 0, 4, 0, 1 );
$notebook->show();

######################################
# Numerical
######################################

my $numeric_vbox  = new Gtk::VBox( $false, 5 );
my $numeric_label = new Gtk::Label($str_Numerical);
my $numeric_hbox1 = new Gtk::HBox( $false, 5 );
my $numeric_hbox2 = new Gtk::HBox( $false, 5 );
my $numeric_hbox3 = new Gtk::HBox( $false, 5 );
$notebook->append_page( $numeric_vbox, $numeric_label );

$numeric_vbox->border_width(15);

my $numeric_keep_filenames = new Gtk::Combo();
@keep_filenames_list = (
	$str_Numeric_keep_filenames1, $str_Numeric_keep_filenames2,
	$str_Numeric_keep_filenames3
);
$numeric_keep_filenames->set_popdown_strings(@keep_filenames_list);
$numeric_keep_filenames->set_value_in_list( $true, $false );
$numeric_keep_filenames->entry->set_text($str_Numeric_keep_filenames1);
$numeric_keep_filenames->set_use_arrows_always($true);
$numeric_keep_filenames->width('400');

#
my $numeric_start_label = new Gtk::Label($str_Numeric_start);
my $numeric_start_adj   = new Gtk::Adjustment( 1, 0.0, 9000.0, 1.0, 5.0, 0.0 );
my $numeric_start_spin  = new Gtk::SpinButton( $numeric_start_adj, 0, 0 );
$numeric_start_spin->set_numeric($true);
my $numeric_increment_label = new Gtk::Label($str_Numeric_increment);
my $numeric_increment_adj   =
  new Gtk::Adjustment( 1, 1.0, 1000.0, 1.0, 1.0, 0.0 );
my $numeric_increment_spin =
  new Gtk::SpinButton( $numeric_increment_adj, 0, 0 );
$numeric_increment_spin->set_numeric($true);
my $numeric_zero_label    = new Gtk::Label("   ");
my $numeric_zero_autofill = new Gtk::CheckButton($str_Numeric_zero_autofill);
$numeric_zero_autofill->set_active($true);

#
my $numeric_prefix_label = new Gtk::Label($str_Numeric_prefix);
my $numeric_prefix_entry = new Gtk::Entry(254);
$numeric_prefix_entry->width('100');
my $numeric_suffix_label = new Gtk::Label($str_Numeric_suffix);
my $numeric_suffix_entry = new Gtk::Entry(254);
$numeric_suffix_entry->width('100');

#$numeric_vbox->pack_start( $numeric_keep_filenames, $false, $false, 5 );
$numeric_vbox->pack_start( $numeric_hbox1,            $false, $false, 4 );
$numeric_vbox->pack_start( $numeric_hbox2,            $false, $false, 4 );
$numeric_vbox->pack_start( $numeric_hbox3,            $false, $false, 4 );
$numeric_hbox1->pack_start( $numeric_start_label,     $false, $false, 0 );
$numeric_hbox1->pack_start( $numeric_start_spin,      $false, $false, 0 );
$numeric_hbox1->pack_start( $numeric_increment_label, $false, $false, 0 );
$numeric_hbox1->pack_start( $numeric_increment_spin,  $false, $false, 0 );
$numeric_hbox1->pack_start( $numeric_zero_label,      $false, $false, 0 );
$numeric_hbox1->pack_start( $numeric_zero_autofill,   $false, $false, 0 );
$numeric_hbox2->pack_start( $numeric_prefix_label,    $false, $false, 0 );
$numeric_hbox2->pack_start( $numeric_prefix_entry,    $false, $false, 0 );
$numeric_hbox2->pack_start( $numeric_suffix_label,    $false, $false, 0 );
$numeric_hbox2->pack_start( $numeric_suffix_entry,    $false, $false, 0 );
$numeric_hbox3->pack_start( $numeric_keep_filenames,  $false, $false, 0 );

$numeric_start_label->show();
$numeric_start_spin->show();
$numeric_increment_label->show();
$numeric_increment_spin->show();
$numeric_zero_label->show();
$numeric_zero_autofill->show();
$numeric_prefix_label->show();
$numeric_prefix_entry->show();
$numeric_suffix_label->show();
$numeric_suffix_entry->show();
$numeric_keep_filenames->show();
$numeric_vbox->show();
$numeric_hbox1->show();
$numeric_hbox2->show();
$numeric_hbox3->show();

######################################
# Replace / Remove
######################################

my $reprem_vbox1 = new Gtk::VBox( $false, 0 );
my $reprem_hbox  = new Gtk::HBox( $false, 5 );
my $reprem_vbox2 = new Gtk::VBox( $false, 0 );
my $reprem_label = new Gtk::Label($str_ReplaceRemove);
$notebook->append_page( $reprem_vbox1, $reprem_label );

$reprem_vbox1->pack_start( $reprem_hbox,  $false, $false, 0 );
$reprem_vbox1->pack_start( $reprem_vbox2, $false, $false, 5 );

$reprem_vbox1->border_width(11);
$reprem_hbox->border_width(6);

my $replace_this_label = new Gtk::Label($str_Replace);
my $replace_this_entry = new Gtk::Entry(99);
$replace_this_entry->width('150');
my $replace_with_label = new Gtk::Label($str_Replace_With);
my $replace_with_entry = new Gtk::Entry(99);
$replace_with_entry->width('150');
my $replace_opt_case  = new Gtk::CheckButton($str_Replace_case);
my $replace_opt_regex = new Gtk::CheckButton($str_Replace_regex);

$reprem_hbox->pack_start( $replace_this_label, $false, $false, 0 );
$reprem_hbox->pack_start( $replace_this_entry, $false, $false, 0 );
$reprem_hbox->pack_start( $replace_with_label, $false, $false, 0 );
$reprem_hbox->pack_start( $replace_with_entry, $false, $false, 0 );
$reprem_vbox2->pack_start( $replace_opt_case,  $false, $false, 0 );
$reprem_vbox2->pack_start( $replace_opt_regex, $false, $false, 0 );

$replace_this_label->show();
$replace_this_entry->show();
$replace_with_label->show();
$replace_with_entry->show();
$replace_opt_case->show();
$replace_opt_regex->show();
$reprem_vbox1->show();
$reprem_hbox->show();
$reprem_vbox2->show();
$reprem_label->show();

######################################
# Insert / Delete
######################################

my $insdel_vbox  = new Gtk::VBox( $false, 5 );
my $insdel_hbox1 = new Gtk::HBox( $false, 5 );
my $insdel_hbox2 = new Gtk::HBox( $false, 5 );
my $insdel_label = new Gtk::Label($str_InsertDelete);
$notebook->append_page( $insdel_vbox, $insdel_label );

$insdel_vbox->border_width(11);

$insdel_vbox->pack_start( $insdel_hbox1, $false, $false, 0 );
$insdel_vbox->pack_start( $insdel_hbox2, $false, $false, 5 );

my $delete_radio     = new Gtk::RadioButton( $str_Delete, );
my $delete_btw1_adj  = new Gtk::Adjustment( 1, -256.0, 256.0, 1.0, 5.0, 0.0 );
my $delete_btw1_spin = new Gtk::SpinButton( $delete_btw1_adj, 0, 0 );
$delete_btw1_spin->set_numeric($true);
my $delete_btw2_label = new Gtk::Label($str_Delete_btw2);
my $delete_btw2_adj   = new Gtk::Adjustment( 1, -256.0, 256.0, 1.0, 5.0, 0.0 );
my $delete_btw2_spin  = new Gtk::SpinButton( $delete_btw2_adj, 0, 0 );
$delete_btw2_spin->set_numeric($true);
my $insert_radio = new Gtk::RadioButton( $str_Insert, $delete_radio );
my $insert_entry = new Gtk::Entry(254);
$insert_entry->width('260');
my $insert_at_label = new Gtk::Label($str_Insert_at);
my $insert_adj      = new Gtk::Adjustment( 1, -256.0, 256.0, 1.0, 5.0, 0.0 );
my $insert_spin     = new Gtk::SpinButton( $insert_adj, 0, 0 );
$insert_spin->set_numeric($true);

$insdel_hbox1->pack_start( $insert_radio,      $false, $false, 2 );
$insdel_hbox1->pack_start( $insert_entry,      $false, $false, 2 );
$insdel_hbox1->pack_start( $insert_at_label,   $false, $false, 2 );
$insdel_hbox1->pack_start( $insert_spin,       $false, $false, 2 );
$insdel_hbox2->pack_start( $delete_radio,      $false, $false, 2 );
$insdel_hbox2->pack_start( $delete_btw1_spin,  $false, $false, 2 );
$insdel_hbox2->pack_start( $delete_btw2_label, $false, $false, 2 );
$insdel_hbox2->pack_start( $delete_btw2_spin,  $false, $false, 2 );

$delete_radio->show();
$delete_btw1_spin->show();
$delete_btw2_label->show();
$delete_btw2_spin->show();
$insert_radio->show();
$insert_entry->show();
$insert_at_label->show();
$insert_spin->show();
$insdel_vbox->show();
$insdel_hbox1->show();
$insdel_hbox2->show();
$insdel_label->show();

######################################
# Case Change
######################################

my $case_hbox = new Gtk::HBox( $false, 0 );
my $case_label = new Gtk::Label($str_CaseChange);
$notebook->append_page( $case_hbox, $case_label );

my $case_vbox1 = new Gtk::VBox( $false, 5 );
my $case_vbox2 = new Gtk::VBox( $false, 13 );

#Yes the spacing here must be 13, it should be 5 but with 13 everything is align perfectly
#must be because of the empty $fillspace, it's odd o_O

$case_hbox->border_width(5);
$case_vbox1->border_width(5);
$case_vbox2->border_width(5);

$case_hbox->pack_start( $case_vbox1, $false, $false, 5 );
$case_hbox->pack_start( $case_vbox2, $false, $false, 5 );

$case_all_up_radio = new Gtk::RadioButton($str_Case_all_up);
$case_all_lo_radio =
  new Gtk::RadioButton( $str_Case_all_lo, $case_all_up_radio );
$case_first_radio = new Gtk::RadioButton( $str_Case_first, $case_all_up_radio );
$case_first_after_entry = new Gtk::Entry(99);
$case_first_after_entry->set_text(' _-');
$case_first_after_entry->width('60');

#Here's the empty label to align the Entry with the 3rd radio button
my $fillspace1 = new Gtk::Label('');
my $fillspace2 = new Gtk::Label('');

$case_vbox1->pack_start( $case_all_up_radio,      $false, $false, 0 );
$case_vbox1->pack_start( $case_all_lo_radio,      $false, $false, 0 );
$case_vbox1->pack_start( $case_first_radio,       $false, $false, 0 );
$case_vbox2->pack_start( $fillspace1,             $false, $false, 0 );
$case_vbox2->pack_start( $fillspace2,             $false, $false, 0 );
$case_vbox2->pack_start( $case_first_after_entry, $false, $false, 0 );

$case_hbox->show();
$case_vbox1->show();
$case_vbox2->show();
$case_all_up_radio->show();
$case_all_lo_radio->show();
$case_first_radio->show();
$case_first_after_entry->show();
$fillspace1->show();
$fillspace2->show();

######################################
# Buttons
######################################

my $button_box = new Gtk::VButtonBox();
$button_box->set_spacing_default(15);
$button_box->set_layout_default('spread');
$table->attach_defaults( $button_box, 4, 5, 0, 1 );
$button_box->show();

my $preview_button = new Gtk::Button($str_Preview);
$button_box->add($preview_button);
$preview_button->signal_connect( "clicked", \&preview );
$preview_button->show();

my $apply_button = new Gtk::Button($str_Apply);
$button_box->add($apply_button);
$apply_button->signal_connect( "clicked", \&apply );
$apply_button->show();

my $undo_button = new Gtk::Button($str_Undo);
$button_box->add($undo_button);
$undo_button->set_sensitive($false);
$undo_button->signal_connect( "clicked", \&undo );
$undo_button->show();

my $tooltips = new Gtk::Tooltips();
$tooltips->set_tip( $numeric_zero_autofill, $str_tool_zero,    "" );
$tooltips->set_tip( $preview_button,        $str_tool_preview, "" );
$tooltips->set_tip( $apply_button,          $str_tool_apply,   "" );
$tooltips->set_tip( $undo_button,           $str_tool_undo,    "" );

main Gtk;
exit(0);

######################################
#           Subroutines
######################################

sub quit {
	&save_settings;
	Gtk->exit(0);
}

sub read_settings {
	if ( !-e $setting_file ) {
		return;
	}
	
	open( INPUT, "<", $setting_file )
	  or die "Couldn't open $setting_file for reading: $!\n";

	flock( INPUT, LOCK_EX );
	
	while (my $line = <INPUT>) {
		chomp($line);
		(my $key, my $value) = split(/=/, $line);
		$settings{$key} = $value;
	}
	
	flock( INPUT, LOCK_UN );
	close( INPUT );
}

sub save_settings {
	open( OUTPUT, ">", $setting_file )
	  or die "Couldn't open $setting_file for writing: $!\n";
	
	flock( OUTPUT, LOCK_EX );
	
	foreach $key ( sort keys %settings ) {
		print OUTPUT $key, '=', $settings{$key}, "\n";
	}
	
	flock( OUTPUT, LOCK_UN );
	close( OUTPUT );
}

sub language_selected {
	my ( $item, $lang ) = @_;
	if ( $item->active ) {
		if ($lang ne $settings{'lang'}) {
			$settings{'lang'} = $lang;
			&prompt_restart;
		}
	}
}

sub prompt_restart {
	if ($prompt_restart_dialog == undef) {
		&create_prompt_restart;
	}
	$prompt_restart_dialog->show();
}

sub create_prompt_restart {
	$dialog = new Gtk::Dialog();
	$dialog->set_parent_window($window->window);
	$dialog->show();
	$dialog->window->set_icon( $dialog->window, $pixmap, $mask );
	$dialog->set_modal($true);
	$dialog->set_policy( $false, $false, $true );
	$dialog->set_position('center');
	$dialog->set_title( "Information" );
	$dialog->border_width(10);
	$button = new Gtk::Button( " Ok " );
	$dialog->action_area->pack_start( $button, $false, $false, 0 );
	$button->signal_connect( 'clicked', sub { $dialog->hide(); } );
	$button->show();
	$button->grab_focus();

	$label = new Gtk::Label(
		'
Please restart GPRename.
'
	);
	$label->set_justify('left');
	$dialog->vbox->pack_start( $label, $false, $false, 0 );
	$label->show();

	$prompt_restart_dialog = $dialog;
}

sub help_about {
	if ($about_dialog == undef) {
		&create_help_about;
	}
	$about_dialog->show();
}

sub create_help_about {
	$about_dialog = new Gtk::Dialog();
	$about_dialog->set_parent_window($window->window);
	$about_dialog->show();
	$about_dialog->window->set_icon( $about_dialog->window, $pixmap, $mask );
	$about_dialog->set_modal($true);
	$about_dialog->set_policy( $false, $false, $true );
	$about_dialog->set_position('center');
	$about_dialog->set_title( $str_About . " - GPRename" );
	$about_dialog->border_width(10);
	$about_button = new Gtk::Button( " " . $str_Close . " " );
	$about_dialog->action_area->pack_start( $about_button, $false, $false, 0 );
	$about_button->signal_connect( 'clicked', sub { $about_dialog->hide(); } );
	$about_button->show();
	$about_button->grab_focus();

	$about_label = new Gtk::Label(
		'GPRename 1.24

Copyright (C) 2001-2004
Tristesse <tristesse@users.sourceforge.net>
Zurd <zurd@users.sourceforge.net>

License:

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'
	);
	$about_label->set_justify('left');
	$about_dialog->vbox->pack_start( $about_label, $false, $false, 0 );
	$about_label->show();
}

# Callback for expanding a tree - find subdirectories and add them to
# tree
sub expand_tree {
	my ( $item, $subtree ) = @_;

	my $dir;
	my $dir_entry;
	my $path;
	my $item_new;
	my $new_subtree;

	$dir = $item->get_user_data();

	chdir($dir);

	foreach $dir_entry (<*>) {
		if ( -d $dir_entry ) {
			$path = $dir . "/" . $dir_entry;
			$path =~ s|//|/|g;
			$item_new = new_with_label Gtk::TreeItem($dir_entry);
			$item_new->set_user_data($path);
			$item_new->signal_connect( 'select', \&select_item, $path );
			$subtree->append($item_new);
			$item_new->show();

			if ( $initial_expand == 1 && $path eq $start_dir ) {
				$item_new->select();
				$autoselected_item = $item_new;
			}

			if ( has_sub_trees($path) ) {
				$new_subtree = new Gtk::Tree();
				$item_new->set_subtree($new_subtree);
				$item_new->signal_connect( 'expand', \&expand_tree,
					$new_subtree );
				$item_new->signal_connect( 'collapse', \&collapse_tree );

				if (   $initial_expand > 0
					&& index($start_dir, $path) == 0 )
				{
					$initial_expand = $initial_expand - 1;
					$item_new->expand();
				}

			}
		}
	}

	chdir("..");
}

# Callback for collapsing a tree -- removes the subtree
sub collapse_tree {
	my ($item) = @_;

	my $subtree = new Gtk::Tree();

	$item->remove_subtree();
	$item->set_subtree($subtree);
	$item->signal_connect( 'expand', \&expand_tree, $subtree );
}

# Test whether a directory has subdirectories
sub has_sub_trees {
	my ($dir) = @_;
	my $file;

	opendir( DIR, $dir );
	my @files = readdir(DIR);
	closedir DIR;

	foreach $file (@files) {
		if ( $file =~ /\A\./ ) { next; }

		$file = $dir . "/" . $file;
		$file =~ s|//|/|g;
		return $true if ( -d $file );
	}

	return ($false);
}

# Called whenever an item is clicked on the tree widget.
sub select_item {
	if ($autoselected_item != undef) {
		$autoselected_item->deselect();
		$autoselected_item = undef;
	}
	my ( $widget, $path ) = @_;
	show_files($path);
	%new_names = ();
}

# Show all the files in a path.
sub show_files {
	($path) = @_;

	my $file;
	my $newname;

	$list->clear();
	$list2->clear();

	opendir( DIR, $path );
	my @files = sort { uc($a) cmp uc($b) } readdir(DIR);
	closedir DIR;

	for $file (@files) {
		if ( $file =~ /\A\./ ) { next; }

		$file = $path . "/" . $file;
		$file =~ s|//|/|g;

		if ( -d $file ) {

			$file =~ s|/.*/||g;
			$file =~ s/\A\///;

			$list2->append( $file, $newname );
		}
		else {

			$file =~ s|/.*/||g;
			$file =~ s/\A\///;

			$list->append( $file, $newname );
		}
	}
}

sub preview {
	%files_to_rename = ();
	%new_names       = ();

	my $page_file_or_dir = $notebook2->get_current_page();
	if ( $page_file_or_dir == 0 ) {    #if file tab
		                               #$list2->clear();
		$all_files_num = $list->rows();
		for ( $i = 0 ; $i < $all_files_num ; $i++ ) {
			$list->set_text( $i, 1, '' );
		}

		my @selects = $list->selection();
		if (@selects) {
			foreach (@selects) {
				$files_to_rename{$_} = $list->get_text( $_, 0 );
			}
		}
		else {
			for ( $i = 0 ; $i < $all_files_num ; $i++ ) {
				$files_to_rename{$i} = $list->get_text( $i, 0 );
			}
		}
	}
	elsif ( $page_file_or_dir == 1 ) {    #if directory tab
		                                  #$list->clear();
		$all_files_num = $list2->rows();
		for ( $i = 0 ; $i < $all_files_num ; $i++ ) {
			$list2->set_text( $i, 1, '' );
		}

		my @selects = $list2->selection();
		if (@selects) {
			foreach (@selects) {
				$files_to_rename{$_} = $list2->get_text( $_, 0 );
			}
		}
		else {
			for ( $i = 0 ; $i < $all_files_num ; $i++ ) {
				$files_to_rename{$i} = $list2->get_text( $i, 0 );
			}
		}
	}

	my $page = $notebook->get_current_page();
	if    ( $page == 0 ) { &numeric; }
	elsif ( $page == 1 ) { &replace; }
	elsif ( $page == 2 ) { &insdel; }
	elsif ( $page == 3 ) { &case }
}

sub apply {

	if ( %files_to_rename && %new_names ) {
		%undo = ();
		my $name_o;
		my $name_n;
		my $name_tmp;
		my $chk;
		my $chk_result = 0;

		foreach $chk ( keys(%new_names) ) {
			if ($chk_result) { last; }
			foreach ( keys(%new_names) ) {
				if ( $chk ne $_ && $new_names{$chk} eq $new_names{$_} ) {
					$chk_result = 1;
					last;
				}
			}
		}

		if ( !$chk_result ) {
			foreach $keyi ( keys(%new_names) ) {
				if ( $new_names{$keyi} ne $files_to_rename{$keyi} ) {
					$name_o   = $path . "/" . $files_to_rename{$keyi};
					$name_tmp =
					  $path . "/" . $new_names{$keyi} . "_tmp_gprename";
					$name_n = $path . "/" . $new_names{$keyi};

					rename( $name_o,   $name_tmp );
					rename( $name_tmp, $name_n );

					$undo{$name_o} = $name_n;
				}
			}

			%files_to_rename = ();
			%new_names       = ();
			$undo_button->set_sensitive($true);
			&show_files($path);
		}
	}
}

sub undo {
	foreach $keyi ( keys(%undo) ) {
		$name_o   = $undo{$keyi};
		$name_tmp = $keyi . "gprename_tmp";
		$name_n   = $keyi;
		rename( $name_o,   $name_tmp );
		rename( $name_tmp, $name_n );
	}

	%undo = ();
	$undo_button->set_sensitive($false);
	&show_files($path);
}

sub numeric {
	my $numeric_start     = $numeric_start_spin->get_value_as_int();
	my $numeric_increment = $numeric_increment_spin->get_value_as_int();
	my $numeric_prefix    = $numeric_prefix_entry->get_chars( 0, -1 );
	my $numeric_suffix    = $numeric_suffix_entry->get_chars( 0, -1 );

	my $temp_rename_num = keys %files_to_rename;
	my $max_no = ( $temp_rename_num - 1 ) * $numeric_increment + $numeric_start;
	my $max_chrs = length($max_no);

	$i = $numeric_start;
	foreach $keyi ( sort { $a <=> $b } keys(%files_to_rename) ) {
		$new_names{$keyi} = $i;
		$i = $i + $numeric_increment;

		if ( $numeric_zero_autofill->active ) {
			while ( length( $new_names{$keyi} ) < $max_chrs ) {
				$new_names{$keyi} = '0' . $new_names{$keyi};
			}
		}

		$new_names{$keyi} =
		  "$numeric_prefix" . "$new_names{$keyi}" . "$numeric_suffix";

		if ( $numeric_keep_filenames->entry->get_text() eq
			$str_Numeric_keep_filenames2 )
		{
			$new_names{$keyi} = "$files_to_rename{$keyi}" . "$new_names{$keyi}";
		}
		if ( $numeric_keep_filenames->entry->get_text() eq
			$str_Numeric_keep_filenames3 )
		{
			$new_names{$keyi} = "$new_names{$keyi}" . "$files_to_rename{$keyi}";
		}

		#In case there's a blank space at the end, let's remove it :
		$new_names{$keyi} =~ s/^\s+//;
		$new_names{$keyi} =~ s/\s+$//;

		my $page_file_or_dir = $notebook2->get_current_page();
		if ( $page_file_or_dir == 0 ) {    #if file tab
			$list->set_text( $keyi, 1, "$new_names{$keyi}" );
		}
		elsif ( $page_file_or_dir == 1 ) {    #if directory tab
			$list2->set_text( $keyi, 1, "$new_names{$keyi}" );
		}

	}
}

sub replace {
	my $replace_this = $replace_this_entry->get_chars( 0, -1 );
	my $replace_with = $replace_with_entry->get_chars( 0, -1 );

	if ($replace_this) {
		foreach $keyi ( keys(%files_to_rename) ) {
			$new_names{$keyi} = $files_to_rename{$keyi};

			if ( $replace_opt_regex->active ) {
				my %temp;
				my $with = $replace_with;

				$files_to_rename{$keyi} =~ /$replace_this/;

				$temp{1} = $1;
				$temp{2} = $2;
				$temp{3} = $3;
				$temp{4} = $4;
				$temp{5} = $5;
				$temp{6} = $6;
				$temp{7} = $7;
				$temp{8} = $8;
				$temp{9} = $9;

				$with =~ s/\$([0-9])/$temp{$1}/g;

				if ( $replace_opt_case->active ) {
					$new_names{$keyi} =~ s/$replace_this/$with/g;
				}
				else { $new_names{$keyi} =~ s/$replace_this/$with/ig; }
			}
			else {
				if ( $replace_opt_case->active ) {
					$new_names{$keyi} =~ s/\Q$replace_this/$replace_with/g;
				}
				else {
					$new_names{$keyi} =~ s/\Q$replace_this/$replace_with/ig;
				}
			}

			my $page_file_or_dir = $notebook2->get_current_page();
			if ( $page_file_or_dir == 0 ) {    #if file tab
				$list->set_text( $keyi, 1, "$new_names{$keyi}" );
			}
			elsif ( $page_file_or_dir == 1 ) {    #if directory tab
				$list2->set_text( $keyi, 1, "$new_names{$keyi}" );
			}
		}
	}
}

sub insdel {
	my $insert_str;
	my $insert_at;
	my $delete_from;
	my $delete_to;
	my $keyi;
	my @temp;

	if ( $insert_radio->active ) {
		$insert_str = $insert_entry->get_chars( 0, -1 );
		$insert_at  = $insert_spin->get_value_as_int();

		if ( $insert_at == 0 ) { $insert_at = 1; }

		foreach $keyi ( keys(%files_to_rename) ) {
			@temp = split( /\s{0}/, $files_to_rename{$keyi} );

			if ( $insert_at > 0 ) {
				if ( $insert_at > @temp ) {
					$new_names{$keyi} = $files_to_rename{$keyi} . $insert_str;
				}
				else {
					for ( $i = 1 ; $i <= @temp ; $i++ ) {
						if ( $insert_at == $i ) {
							$new_names{$keyi} =
							  $new_names{$keyi} . $insert_str . $temp[ $i - 1 ];
						}
						else {
							$new_names{$keyi} =
							  $new_names{$keyi} . $temp[ $i - 1 ];
						}
					}
				}
			}

			else {
				my $temp_insert_at = $insert_at * -1;
				if ( $temp_insert_at > @temp ) {
					$new_names{$keyi} = $insert_str . $files_to_rename{$keyi};
				}
				else {
					@temp = reverse(@temp);
					for ( $i = 1 ; $i <= @temp ; $i++ ) {
						if ( $temp_insert_at == $i ) {
							$new_names{$keyi} =
							  $temp[ $i - 1 ] . $insert_str . $new_names{$keyi};
						}
						else {
							$new_names{$keyi} =
							  $temp[ $i - 1 ] . $new_names{$keyi};
						}
					}
				}
			}

			my $page_file_or_dir = $notebook2->get_current_page();
			if ( $page_file_or_dir == 0 ) {    #if file tab
				$list->set_text( $keyi, 1, "$new_names{$keyi}" );
			}
			elsif ( $page_file_or_dir == 1 ) {    #if directory tab
				$list2->set_text( $keyi, 1, "$new_names{$keyi}" );
			}
		}

	}
	elsif ( $delete_radio->active ) {
		$delete_btw1 = $delete_btw1_spin->get_value_as_int();
		$delete_btw2 = $delete_btw2_spin->get_value_as_int();
		if ( $delete_btw1 == 0 ) { $delete_btw1 = 1; }
		if ( $delete_btw2 == 0 ) { $delete_btw2 = 1; }

		foreach $keyi ( keys(%files_to_rename) ) {
			@temp = split( /\s{0}/, $files_to_rename{$keyi} );

			my $temp_btw1 = $delete_btw1;
			my $temp_btw2 = $delete_btw2;
			if ( $delete_btw1 > @temp )      { $temp_btw1 = @temp; }
			if ( $delete_btw1 * -1 > @temp ) { $temp_btw1 = @temp * -1; }
			if ( $delete_btw2 > @temp )      { $temp_btw2 = @temp; }
			if ( $delete_btw2 * -1 > @temp ) { $temp_btw2 = @temp * -1; }

			if ( $temp_btw1 < 0 ) { $temp_btw1 = $temp_btw1 + @temp + 1; }
			if ( $temp_btw2 < 0 ) { $temp_btw2 = $temp_btw2 + @temp + 1; }

			my $max;
			my $min;
			if ( $temp_btw1 >= $temp_btw2 ) {
				$max = $temp_btw1;
				$min = $temp_btw2;
			}
			else { $max = $temp_btw2; $min = $temp_btw1; }

			for ( $i = 1 ; $i <= @temp ; $i++ ) {
				if ( $i < $min || $i > $max ) {
					$new_names{$keyi} = $new_names{$keyi} . $temp[ $i - 1 ];
				}
			}

			my $page_file_or_dir = $notebook2->get_current_page();
			if ( $page_file_or_dir == 0 ) {    #if file tab
				$list->set_text( $keyi, 1, "$new_names{$keyi}" );
			}
			elsif ( $page_file_or_dir == 1 ) {    #if directory tab
				$list2->set_text( $keyi, 1, "$new_names{$keyi}" );
			}
		}
	}
}

sub case {
	my $case_first_after = $case_first_after_entry->get_chars( 0, -1 );
	my @case_first_after = split( /\s{0}/, $case_first_after );

	foreach $keyi ( keys(%files_to_rename) ) {
		if ( $case_all_up_radio->active ) {
			$new_names{$keyi} = "\U$files_to_rename{$keyi}";
		}
		elsif ( $case_all_lo_radio->active ) {
			$new_names{$keyi} = "\L$files_to_rename{$keyi}";
		}
		else {
			my $temp = "\L\u$files_to_rename{$keyi}";
			foreach (@case_first_after) {
				my @temp = split( /\Q$_/, $temp );
				foreach $frag (@temp) { $frag = "\u$frag"; }
				$temp = join( "$_", @temp );

		   #If this is enabled, gprename will add a dot at the end of a filename
		   #if a user insert a dot in the first letter text field :
		   #if ( $files_to_rename{$keyi} =~ /$_\Z/ ){ $temp = $temp . $_; }
			}
			$new_names{$keyi} = $temp;
		}
		my $page_file_or_dir = $notebook2->get_current_page();
		if ( $page_file_or_dir == 0 ) {    #if file tab
			$list->set_text( $keyi, 1, "$new_names{$keyi}" );
		}
		elsif ( $page_file_or_dir == 1 ) {    #if directory tab
			$list2->set_text( $keyi, 1, "$new_names{$keyi}" );
		}
	}
}
